@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2021 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -17,8 +17,8 @@
 //
 // Alternative licensing terms are available from the licensor.
 // For commercial licensing, see <https://www.artifex.com/> or contact
-// Artifex Software, Inc., 1305 Grant Avenue - Suite 200, Novato,
-// CA 94945, U.S.A., +1(415)492-9861, for further information.
+// Artifex Software, Inc., 39 Mesa Street, Suite 108A, San Francisco,
+// CA 94129, USA, for further information.
 
 #include "mupdf/fitz.h"
 
@@ -88,15 +88,6 @@ static inline int iswhite(int a)
 	return 0;
 }
 
-static inline int iseol(int a)
-{
-	switch (a) {
-	case '\r': case '\n':
-		return 1;
-	}
-	return 0;
-}
-
 static inline int bitdepth_from_maxval(int maxval)
 {
 	int depth = 0;
@@ -112,9 +103,9 @@ static const unsigned char *
 pnm_read_signature(fz_context *ctx, const unsigned char *p, const unsigned char *e, char *signature)
 {
 	if (e - p < 2)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse magic number in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse magic number in pnm image");
 	if (p[0] != 'P' || ((p[1] < '1' || p[1] > '7') && p[1] != 'F' && p[1] != 'f'))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected signature in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected signature in pnm image");
 
 	signature[0] = *p++;
 	signature[1] = *p++;
@@ -125,7 +116,7 @@ static const unsigned char *
 pnm_read_until_eol(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse line in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse line in pnm image");
 
 	while (p < e && ((acceptCR && *p != '\r' && *p != '\n') || (!acceptCR && *p != '\n')))
 		p++;
@@ -137,9 +128,9 @@ static const unsigned char *
 pnm_read_eol(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse end of line in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse end of line in pnm image");
 	if ((acceptCR && *p != '\r' && *p != '\n') || (!acceptCR && *p != '\n'))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected end of line in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected end of line in pnm image");
 
 	/* CR, CRLF or LF depending on acceptCR. */
 	if (acceptCR && *p == '\r')
@@ -154,9 +145,9 @@ static const unsigned char *
 pnm_read_whites(fz_context *ctx, const unsigned char *p, const unsigned char *e, int required)
 {
 	if (required && e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse whitespaces in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse whitespaces in pnm image");
 	if (required && !iswhite(*p))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected whitespaces in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected whitespaces in pnm image");
 
 	while (p < e && iswhite(*p))
 		p++;
@@ -168,9 +159,9 @@ static const unsigned char *
 pnm_read_white_or_eol(fz_context *ctx, const unsigned char *p, const unsigned char *e)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse whitespace/eol in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse whitespace/eol in pnm image");
 	if (!iswhiteeol(*p))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected whitespace/eol in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected whitespace/eol in pnm image");
 
 	return ++p;
 }
@@ -179,9 +170,9 @@ static const unsigned char *
 pnm_read_whites_and_eols(fz_context *ctx, const unsigned char *p, const unsigned char *e, int required)
 {
 	if (required && e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse whitespaces/eols in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse whitespaces/eols in pnm image");
 	if (required && !iswhiteeol(*p))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected whitespaces/eols in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected whitespaces/eols in pnm image");
 
 	while (p < e && iswhiteeol(*p))
 		p++;
@@ -193,7 +184,7 @@ static const unsigned char *
 pnm_read_comment(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse line in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse line in pnm image");
 
 	if (*p != '#')
 		return p;
@@ -205,7 +196,7 @@ static const unsigned char *
 pnm_read_comments(fz_context *ctx, const unsigned char *p, const unsigned char *e, int acceptCR)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse comment in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse comment in pnm image");
 
 	while (p < e && *p == '#')
 	{
@@ -220,9 +211,9 @@ static const unsigned char *
 pnm_read_digit(fz_context *ctx, const unsigned char *p, const unsigned char *e, int *number)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse digit in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse digit in pnm image");
 	if (*p < '0' || *p > '1')
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected digit in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected digit in pnm image");
 
 	if (number)
 		*number = *p - '0';
@@ -235,9 +226,9 @@ static const unsigned char *
 pnm_read_int(fz_context *ctx, const unsigned char *p, const unsigned char *e, int *number)
 {
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse integer in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse integer in pnm image");
 	if (*p < '0' || *p > '9')
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected integer in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected integer in pnm image");
 
 	while (p < e && *p >= '0' && *p <= '9')
 	{
@@ -257,10 +248,10 @@ pnm_read_real(fz_context *ctx, const unsigned char *p, const unsigned char *e, f
 	size_t len;
 
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse real in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse real in pnm image");
 
 	if (*p != '+' && *p != '-' && (*p < '0' || *p > '9'))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "expected numeric field in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "expected numeric field in pnm image");
 
 	while (p < e && (*p == '+' || *p == '-' || *p == '.' || (*p >= '0' && *p <= '9')))
 		p++;
@@ -301,10 +292,10 @@ pnm_read_tupletype(fz_context *ctx, const unsigned char *p, const unsigned char
 	int i, len;
 
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse tuple type in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse tuple type in pnm image");
 
 	s = p;
-	while (!iswhiteeol(*p))
+	while (p < e && !iswhiteeol(*p))
 		p++;
 	len = p - s;
 
@@ -315,7 +306,7 @@ pnm_read_tupletype(fz_context *ctx, const unsigned char *p, const unsigned char
 			return p;
 		}
 
-	fz_throw(ctx, FZ_ERROR_GENERIC, "unknown tuple type in pnm image");
+	fz_throw(ctx, FZ_ERROR_FORMAT, "unknown tuple type in pnm image");
 }
 
 static const unsigned char *
@@ -334,10 +325,10 @@ pnm_read_token(fz_context *ctx, const unsigned char *p, const unsigned char *e,
 	int i, len;
 
 	if (e - p < 1)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "cannot parse header token in pnm image");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "cannot parse header token in pnm image");
 
 	s = p;
-	while (!iswhiteeol(*p))
+	while (p < e && !iswhiteeol(*p))
 		p++;
 	len = p - s;
 
@@ -348,7 +339,7 @@ pnm_read_token(fz_context *ctx, const unsigned char *p, const unsigned char *e,
 			return p;
 		}
 
-	fz_throw(ctx, FZ_ERROR_GENERIC, "unknown header token in pnm image");
+	fz_throw(ctx, FZ_ERROR_FORMAT, "unknown header token in pnm image");
 }
 
 static int
@@ -390,16 +381,16 @@ pnm_ascii_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 	}
 
 	if (pnm->maxval <= 0 || pnm->maxval >= 65536)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "maximum sample value of out range in pnm image: %d", pnm->maxval);
+		fz_throw(ctx, FZ_ERROR_FORMAT, "maximum sample value of out range in pnm image: %d", pnm->maxval);
 
 	pnm->bitdepth = bitdepth_from_maxval(pnm->maxval);
 
 	if (pnm->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (pnm->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
 	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image too large");
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
 
 	if (onlymeta)
 	{
@@ -439,39 +430,48 @@ pnm_ascii_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		int w, h, n;
 
 		img = fz_new_pixmap(ctx, pnm->cs, pnm->width, pnm->height, NULL, 0);
-		dp = img->samples;
 
-		w = img->w;
-		h = img->h;
-		n = img->n;
-
-		if (bitmap)
+		fz_try(ctx)
 		{
-			for (y = 0; y < h; y++)
+			dp = img->samples;
+
+			w = img->w;
+			h = img->h;
+			n = img->n;
+
+			if (bitmap)
 			{
-				for (x = 0; x < w; x++)
+				for (y = 0; y < h; y++)
 				{
-					int v = 0;
-					p = pnm_read_whites_and_eols(ctx, p, e, 0);
-					p = pnm_read_digit(ctx, p, e, &v);
-					p = pnm_read_whites_and_eols(ctx, p, e, 0);
-					*dp++ = v ? 0x00 : 0xff;
-				}
-			}
-		}
-		else
-		{
-			for (y = 0; y < h; y++)
-				for (x = 0; x < w; x++)
-					for (k = 0; k < n; k++)
+					for (x = 0; x < w; x++)
 					{
 						int v = 0;
 						p = pnm_read_whites_and_eols(ctx, p, e, 0);
-						p = pnm_read_int(ctx, p, e, &v);
+						p = pnm_read_digit(ctx, p, e, &v);
 						p = pnm_read_whites_and_eols(ctx, p, e, 0);
-						v = fz_clampi(v, 0, pnm->maxval);
-						*dp++ = map_color(ctx, v, pnm->maxval, 255);
+						*dp++ = v ? 0x00 : 0xff;
 					}
+				}
+			}
+			else
+			{
+				for (y = 0; y < h; y++)
+					for (x = 0; x < w; x++)
+						for (k = 0; k < n; k++)
+						{
+							int v = 0;
+							p = pnm_read_whites_and_eols(ctx, p, e, 0);
+							p = pnm_read_int(ctx, p, e, &v);
+							p = pnm_read_whites_and_eols(ctx, p, e, 0);
+							v = fz_clampi(v, 0, pnm->maxval);
+							*dp++ = map_color(ctx, v, pnm->maxval, 255);
+						}
+			}
+		}
+		fz_catch(ctx)
+		{
+			fz_drop_pixmap(ctx, img);
+			fz_rethrow(ctx);
 		}
 	}
 
@@ -485,6 +485,11 @@ static fz_pixmap *
 pnm_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, const unsigned char *e, int onlymeta, int bitmap, const unsigned char **out)
 {
 	fz_pixmap *img = NULL;
+	size_t span;
+	int n;
+
+	n = fz_colorspace_n(ctx, pnm->cs);
+	assert(n >= 1 && n <= 3);
 
 	pnm->width = 0;
 	p = pnm_read_comments(ctx, p, e, 1);
@@ -513,80 +518,95 @@ pnm_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 	}
 
 	if (pnm->maxval <= 0 || pnm->maxval >= 65536)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "maximum sample value of out range in pnm image: %d", pnm->maxval);
+		fz_throw(ctx, FZ_ERROR_FORMAT, "maximum sample value of out range in pnm image: %d", pnm->maxval);
 
 	pnm->bitdepth = bitdepth_from_maxval(pnm->maxval);
 
 	if (pnm->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (pnm->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
-	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image too large");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
+	if (bitmap)
+	{
+		/* Overly sensitive test, but we can live with it. */
+		if ((size_t)pnm->width > SIZE_MAX / (unsigned int)n)
+			fz_throw(ctx, FZ_ERROR_LIMIT, "image row too large");
+		span = ((size_t)n * pnm->width + 7)/8;
+	}
+	else
+	{
+		size_t bytes_per_sample = (pnm->bitdepth-1)/8 + 1;
+		span = (size_t)n * bytes_per_sample;
+		if ((size_t)pnm->width > SIZE_MAX / span)
+			fz_throw(ctx, FZ_ERROR_LIMIT, "image row too large");
+		span = (size_t)pnm->width * span;
+	}
+	if ((size_t)pnm->height > SIZE_MAX / span)
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
+	if (e - p < 0 || ((size_t)(e - p)) < span * (size_t)pnm->height)
+		fz_throw(ctx, FZ_ERROR_FORMAT, "insufficient data");
 
 	if (onlymeta)
 	{
-		int w = pnm->width;
-		int h = pnm->height;
-		int n = fz_colorspace_n(ctx, pnm->cs);
-
-		if (pnm->maxval == 255)
-			p += n * w * h;
-		else if (bitmap)
-			p += ((w + 7) / 8) * h;
-		else if (pnm->maxval < 255)
-			p += n * w * h;
-		else
-			p += 2 * n * w * h;
+		p += span * (size_t)pnm->height;
 	}
 	else
 	{
 		unsigned char *dp;
 		int x, y, k;
-		int w, h, n;
+		int w, h;
 
 		img = fz_new_pixmap(ctx, pnm->cs, pnm->width, pnm->height, NULL, 0);
-		dp = img->samples;
 
-		w = img->w;
-		h = img->h;
-		n = img->n;
+		fz_try(ctx)
+		{
+			dp = img->samples;
 
-		if (pnm->maxval == 255)
-		{
-			memcpy(dp, p, (size_t)w * h * n);
-			p += n * w * h;
-		}
-		else if (bitmap)
-		{
-			for (y = 0; y < h; y++)
+			w = img->w;
+			h = img->h;
+			n = img->n;
+
+			if (pnm->maxval == 255)
 			{
-				for (x = 0; x < w; x++)
+				memcpy(dp, p, (size_t)w * h * n);
+				p += n * w * h;
+			}
+			else if (bitmap)
+			{
+				for (y = 0; y < h; y++)
 				{
-					*dp++ = (*p & (1 << (7 - (x & 0x7)))) ? 0x00 : 0xff;
-					if ((x & 0x7) == 7)
+					for (x = 0; x < w; x++)
+					{
+						*dp++ = (*p & (1 << (7 - (x & 0x7)))) ? 0x00 : 0xff;
+						if ((x & 0x7) == 7)
+							p++;
+					}
+					if (w & 0x7)
 						p++;
 				}
-				if (w & 0x7)
-					p++;
+			}
+			else if (pnm->maxval < 255)
+			{
+				for (y = 0; y < h; y++)
+					for (x = 0; x < w; x++)
+						for (k = 0; k < n; k++)
+							*dp++ = map_color(ctx, *p++, pnm->maxval, 255);
+			}
+			else
+			{
+				for (y = 0; y < h; y++)
+					for (x = 0; x < w; x++)
+						for (k = 0; k < n; k++)
+						{
+							*dp++ = map_color(ctx, (p[0] << 8) | p[1], pnm->maxval, 255);
+							p += 2;
+						}
 			}
 		}
-		else if (pnm->maxval < 255)
+		fz_catch(ctx)
 		{
-			for (y = 0; y < h; y++)
-				for (x = 0; x < w; x++)
-					for (k = 0; k < n; k++)
-						*dp++ = map_color(ctx, *p++, pnm->maxval, 255);
-		}
-		else
-		{
-			for (y = 0; y < h; y++)
-				for (x = 0; x < w; x++)
-					for (k = 0; k < n; k++)
-					{
-						*dp++ = map_color(ctx, (p[1] << 8) | p[0], pnm->maxval, 255);
-						p += 2;
-					}
+			fz_drop_pixmap(ctx, img);
+			fz_rethrow(ctx);
 		}
 	}
 
@@ -620,7 +640,7 @@ pam_binary_read_header(fz_context *ctx, struct info *pnm, const unsigned char *p
 			p = pnm_read_token(ctx, p, eol, &token);
 
 			if (seen[token - 1])
-				fz_throw(ctx, FZ_ERROR_GENERIC, "token occurs multiple times in pnm image");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "token occurs multiple times in pnm image");
 			seen[token - 1] = 1;
 
 			if (token != TOKEN_ENDHDR)
@@ -669,7 +689,7 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		case 4: pnm->tupletype = PAM_CMYK; break;
 		case 5: pnm->tupletype = PAM_CMYKA; break;
 		default:
-			fz_throw(ctx, FZ_ERROR_GENERIC, "cannot guess tuple type based on depth in pnm image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "cannot guess tuple type based on depth in pnm image");
 		}
 
 	if (pnm->tupletype == PAM_BW && pnm->maxval > 1)
@@ -711,22 +731,22 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		pnm->cs = fz_device_cmyk(ctx);
 		break;
 	default:
-		fz_throw(ctx, FZ_ERROR_GENERIC, "unsupported tuple type");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "unsupported tuple type");
 	}
 
 	if (pnm->depth != fz_colorspace_n(ctx, pnm->cs) + pnm->alpha)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "depth out of tuple type range");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "depth out of tuple type range");
 	if (pnm->maxval < minval || pnm->maxval > maxval)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "maxval out of range");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "maxval out of range");
 
 	pnm->bitdepth = bitdepth_from_maxval(pnm->maxval);
 
 	if (pnm->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (pnm->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
 	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image too large");
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
 
 	if (onlymeta)
 	{
@@ -750,9 +770,9 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 				packed = 1;
 		}
 		if (packed && (e < p || (size_t)(e - p) < size / 8))
-			fz_throw(ctx, FZ_ERROR_GENERIC, "truncated packed image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated packed image");
 		if (!packed && (e < p || (size_t)(e - p) < size * (pnm->maxval < 256 ? 1 : 2)))
-			fz_throw(ctx, FZ_ERROR_GENERIC, "truncated image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
 
 		if (pnm->maxval == 255)
 			p += size;
@@ -793,9 +813,9 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 					packed = 1;
 			}
 			if (packed && (e < p || (size_t)(e - p) < size / 8))
-				fz_throw(ctx, FZ_ERROR_GENERIC, "truncated packed image");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "truncated packed image");
 			if (!packed && (e < p || (size_t)(e - p) < size * (pnm->maxval < 256 ? 1 : 2)))
-				fz_throw(ctx, FZ_ERROR_GENERIC, "truncated image");
+				fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
 
 			if (pnm->maxval == 255)
 				memcpy(dp, p, size);
@@ -834,7 +854,7 @@ pam_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 					for (x = 0; x < w; x++)
 						for (k = 0; k < n; k++)
 						{
-							*dp++ = map_color(ctx, (p[1] << 8) | p[0], pnm->maxval, 255);
+							*dp++ = map_color(ctx, (p[0] << 8) | p[1], pnm->maxval, 255);
 							p += 2;
 						}
 			}
@@ -892,11 +912,11 @@ pfm_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 	pnm->cs = rgb ? fz_device_rgb(ctx) : fz_device_gray(ctx);
 
 	if (pnm->height <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image height must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image height must be > 0");
 	if (pnm->width <= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image width must be > 0");
+		fz_throw(ctx, FZ_ERROR_FORMAT, "image width must be > 0");
 	if ((unsigned int)pnm->height > UINT_MAX / pnm->width / fz_colorspace_n(ctx, pnm->cs) / (pnm->bitdepth / 8 + 1))
-		fz_throw(ctx, FZ_ERROR_GENERIC, "image too large");
+		fz_throw(ctx, FZ_ERROR_LIMIT, "image too large");
 
 	if (onlymeta)
 	{
@@ -906,7 +926,7 @@ pfm_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		size_t size = w * h * n * sizeof(float);
 
 		if (e < p || (size_t)(e - p) < size)
-			fz_throw(ctx, FZ_ERROR_GENERIC, "truncated image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
 
 		p += size;
 	}
@@ -921,7 +941,7 @@ pfm_binary_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p,
 		int x, y, k;
 
 		if (e < p || (size_t)(e - p) < size)
-			fz_throw(ctx, FZ_ERROR_GENERIC, "truncated image");
+			fz_throw(ctx, FZ_ERROR_FORMAT, "truncated image");
 
 		sample = samples = fz_malloc(ctx, size);
 		fz_try(ctx)
@@ -1009,7 +1029,9 @@ pnm_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, size_t
 		else if (!strcmp(signature, "PF"))
 			pix = pfm_binary_read_image(ctx, pnm, p, e, subonlymeta, 1, &p);
 		else
-			fz_throw(ctx, FZ_ERROR_GENERIC, "unsupported portable anymap signature (0x%02x, 0x%02x)", signature[0], signature[1]);
+			fz_throw(ctx, FZ_ERROR_FORMAT, "unsupported portable anymap signature (0x%02x, 0x%02x)", signature[0], signature[1]);
+
+		p = pnm_read_whites_and_eols(ctx, p, e, 0);
 
 		if (onlymeta)
 			pnm->subimages++;
@@ -1018,7 +1040,7 @@ pnm_read_image(fz_context *ctx, struct info *pnm, const unsigned char *p, size_t
 	}
 
 	if (p >= e && subimage >= 0)
-		fz_throw(ctx, FZ_ERROR_GENERIC, "subimage count out of range");
+		fz_throw(ctx, FZ_ERROR_ARGUMENT, "subimage count out of range");
 
 	return pix;
 }
